% data_banco_servocomandi_prj_ASTIB_140bar.m
% M.Sorli


% close all
% clear all
% clc

%% pressure data
ps=14e6;                                                                   %[Pa] nominal supply pressure
pr=1e5;                                                                  %[Pa] nominal return pressure 

%% servoactuator
d_rod=50e-3;                                                               %[m] actuator rod diameter
d_piston=65e-3;                                                            %[m] actuator piston diameter
l=50e-3;                                                                   %[m] half actuator total stroke 
A=pi*(d_piston^2-d_rod^2)/4;                                               %[m^2] piston load section 
m=19;                                                                      %[kg] piston and rod mass 
m_cil=70;                                                                  %[kg] cylinder mass 
Vd=0.1*l*A;                                                                %[m^3] dead volume
Ve=0.004224*A;                                                             %[m^3] Error volume (mounting error from kinematic study)
V0C1_n=l*A+Vd-Ve;                                                          %[m^3] initial actuator volume (chamber 1) 
V0C2_n=l*A+Vd+Ve;                                                          %[m^3] initial actuator volume (chamber 2) 
hcyl=48e-6;                                                                %[m] Radial piston clearance
lm=32e-3 ;                                                                 %[m] Piston length
PC=100e5;                                                                  %[Pa]Initial crossing pressure with no command
c=800;%600;                                                                %[N/(m/s)] viscous damping coefficient  
F_stall=(ps-pr)*A;                                                         % Stall Force [N]

%% calibrated orifice
                                                   
Orifice.l_orifice=10e-3;                                                   %[m] calibrated orifice length
Orifice.d_orifice=1e-3;                                                    %[m] calibrated orifice minimum diameter
Orifice.A_orifice=pi*Orifice.d_orifice^2/4;                                %[m^2] calibrated orifice maximum aera
Orifice.r_orifice=0.1e-4;                                                  %[m] curvature
 Orifice.k_orifice=1+2.677...                                              %[-] parameter to evaluate the discharge coefficient
     *(Orifice.r_orifice/Orifice.d_orifice)-...
     6.04*(Orifice.r_orifice/Orifice.d_orifice)^2;



%% hydraulic parameters

T_oil=40;%42;                                                              %[C] Oil temperature
load('ISOVG32_Prop.mat')                                                   % Hydraulic fluid parameters
beta_nom=interp1(ISO32.bulk(:,1),ISO32.bulk(:,2),T_oil);                   % Oil Bulk module [Pa]
rho=interp1(ISO32.ro_v(:,1),ISO32.ro_v(:,2),T_oil);                        % Oil density [kg/m^3]
mi=interp1(ISO32.visc(:,1),ISO32.visc(:,2),T_oil);                         % Oil Kinematic viscosity [m^2/s]
mu=mi*rho;                                                                 % Oil Dynamic viscosity [Pa*s] 

fr_a=0.008;%0.005;                                                         %[-] volumetric fraction of free air 
beta=1./(1./beta_nom+fr_a/ps);                                             %[Pa] equivalent Bulk modulus

%% Actuator friction
% Stribeck model
F_dynamic=200;                                                             %[N] friction force on actuator seals
F_static=F_dynamic*1.5;                                                    %[N] static friction force on actuator seals 
Fc=F_dynamic;                                                              %[N] Coulomb friction
Fs=F_static;                                                               %[N] Static friction
xs_dot=0.1;                                                                %[m/s] Stribeck speed coefficient
is=5;                                                                      %[-] Stribeck exponent
k_tanh=250;                                                                %[-] hyperbolic tangent coefficient

%% servovalve model parameters
% Servovalve Bosh Rexroth with the following part number:4WSE2EM-6-2X-20-8-ET-315-K17-E-V

Qn=20/60000;                                                               %[m^3/s] Nomunal flow
Cd_v=0.61;                                                                 %[] Servovalve Flow coefficient [-]
DeltaPn=70e5;                                                              %[Pa] Nominal differential pressure DeltaPn= Ps-Pr-DeltaPL
Deltat=4.5e-4;                                                             %[s] Time delay of the conditioning servovalve module (Estimated by a preminary linear model)
i_max=10e-3;                                                               %[A] Maximum servovalve coil current(from Dataforth catalogue)
kcond=1e-3;                                                                %[A/V] static conditioning gain (from Dataforth catalogue)
xs_max=5e-4;                                                               %[m] Bosch maximum stroke
A_vmax=Qn/(Cd_v*sqrt(DeltaPn/rho));                                        %[m^2] Maximum area calculated by using nominal parameters (Qn and DeltaPn)
GM=xs_max/i_max;                                                           % Servovalve mechanical gain [m/A]

% ex Moog 31 parameters
k1_v=0.025*0.112984;                                                       %[m*N/mA]] torque motor gain 
k1_v=k1_v*1000;                                                            %[m*N/A]] torque motor gain 
kf_v=115*0.112984/2.54e-2;                                                  %[m*N/m]] armature/flapper transfer function parameter
k2_v=150*(2.54e-2)^2;                                                      %[m^3/sm] hydraulic amplifierKk2_v gain
A_v=0.026*(2.54e-2)^2;                                                     %[m^2] spool end area
kw_v=16.7*0.112984/2.54e-2;                                                %[m*N/m]] feedback wire gain                              
xs_moog=0.015*2.54e-2;                                                     %[m]Moog maximum spool displacement (from servovalve catalogue) 
If=4.4e-6*0.112984/2.54e-2;                                                %[Nm/(m/s2)] Rotational mass of armature/flapper 
omegan_v=sqrt(kf_v/If);                                                    %[rad/s] Servovalva natural frequency  
zetan_v=0.4;                                                               %[-] Servovalve damping factor 
hrad=3.125e-6;                                                             %[m] radial clearance 
Kx=xs_max/xs_moog;                                                         %[-] Displacement gain

% ULs1=1e-2*xs_max;                                                          %[m] servovalve underlap port s-1 width
% ULr1=0.4e-2*xs_max;                                                        %[m] servovalve underlap port r-1 width
ULs1=0*xs_max;                                                             %[m] servovalve underlap port s-1 width
ULr1=0*xs_max;                                                             %[m] servovalve underlap port r-1 width
ULs2=ULs1;                                                                 %[m] servovalve underlap port s-2 width
ULr2=ULr1;                                                                 %[m] servovalve underlap port r-2 width

ws1=A_vmax/xs_max;                                                         %[m] servovalve port s-1 width 
wr1=ws1;                                                                   %[m] servovalve port r-1 width
ws2=ws1;                                                                   %[m] servovalve port s-2 width
wr2=ws2;                                                                   %[m] servovalve port r-2 width

%% servovalve flux coefficient Cd
Cd_min=0.47;                                                               %[-] minimum damping flow coefficient
Cd_max=0.61;                                                               %[-] maximum damping flow coefficient        
r_Cd=1.1e-6;                                                               %[m] curvature
%% support structure parameters (estimated by SolidWork simulation and fitting)

% St.k_US=3.35e8;                                                            % Upper side rod stiffness [N/m]  
% St.k_LS=0.877e9;                                                           % Lower side rod stiffness [N/m]
% St.c=8e3;                                                                  % Viscous coefficient [N*s/m]
St.k_US=2e7;                                                               % Upper side rod stiffness [N/m]  
St.k_LS=0.1e9;                                                             % Lower side rod stiffness [N/m]
St.c=6e3;                                                                  % Viscous coefficient [N*s/m]
%% force transducer Metior CVS 50kN

kR=2e-4;                                                                   %[V/N] static load cell gain (Dataforth)
LC.m=1.4;                                                                  % Load cell mass [kg]
LC.maxL=50e3;                                                              % Maximum load [N]
LC.maxD=3e-4;                                                              % Maximum deflection [m]
LC.k=LC.maxL/LC.maxD;                                                      % Load cell stiffness [N/m]
LC.zita=0.7;                                                               % Damping factor [-]
LC.c=LC.zita*2*sqrt(LC.k*LC.m);                                            % Viscous coefficient [N*s/m]
LC.S=2e-3;                                                                 %[V/V] Load cell sensitivity
LC.SV=3.3;                                                                 %[V] Bridge supply voltage
LC.SF=3.335;                                                               % [V/V] NI9218 scale factor
LC.KR=LC.maxL*LC.SF/LC.S/LC.SV;                                            %[N/V] Load cell gain (NI 9218)

%% Angular coupling
% Loading kinematics parameters
load('Couple.mat')

%% Control

%  Transport delay ******************************
f_microp=790.4;                                                            % sampling rate, microprocessor recursion rate [Hz]
t_calc=5e-4;                                                               % microprocessor computation time [s]
deltat=1/(2*f_microp)+t_calc; %1e-4;                                       % Digital Control transport delay [s]

%% Control originario
Control.H6=1e-7;                                                           % [1/N]
Control.tauz6=1;                                                           % [s]
Control.taup6=0.09;                                                        % [s]
Control.H1=0.00075;                                                        % [V/N]
Control.tauz1=0.005;                                                       % [s]
Control.taup1=0.050;                                                       % [s]
Control.Gi=0.02;                                                           % [V/Ns] Integrative gain
Control.ks=0.1;                                                            % [-]
Control.H3=0.001;                                                          % [V/N] Propotional gain
Control.tauz3=0.009;                                                       % [s]
Control.taup3=0.022;                                                       % [s]
Control.H4=5e-6;                                                           % [V/N] Derivative gain
Control.tauz4=0.1;                                                         % [s]  
Control.taup4=0.018;                                                       % [s]
Control.H5=18;                                                              % [Vs/m]
Control.taup5=0.00076929;                                                           % [s]
Control.tauz5=7.6929e-5;                                                           % [s]
Control.expy=1.1;                                                          % [-]
Control.deltap=0.5e6;                                                     % [Pa]
Control.ky=0.01;                                                           % [-]
Control.Hs=0;
Control.SV_delay=3e-3;                                                     % [s]  
Control.kv=1;                                                              % [ms/ms]
Control.ka=5e-6;                                                           % [ms^2/ms]
Control.Dead_band=30;                                                      %[N] 
Sample=1/5000;

%% Control
% % linearization gains
% K_A=5e-3;  % [A/N]
% G_T=1e-4;  % [N/N]
% KF=7.89e6; % [N/A}
% bx0=1.86e5;  % [Ns/m]
% GFFgain=bx0/(G_T*K_A*KF); % [Ns/m]
% 
% % X gain *******************************
% Control.H6=1e-7;                                                           % [1/N]
% Control.tauz6=1;                                                           % [s]
% Control.taup6=0.09;                                                        % [s]
% % Control.H6=1e-7;                                                           % [1/N]
% % Control.tauz6=1;                                                           % [s]
% % Control.taup6=0.09;                                                        % [s]
% Control.ks=0.1;                                                            % [-]
% 
% % global LOOP with SET FEEDFORWARD *******************************
% Control.GF=0.48;                                                           % [N/N]
% % Control.H1=Control.GF*G_T*K_A/kcond;                                       % [V/N]
% % Control.tauz1=0.050;                                                       % [s]
% % Control.taup1=0.050;                                                       % [s]
% Control.H1=0.00075;                                                        % [V/N]
% Control.tauz1=0.005;                                                       % [s]
% Control.taup1=0.050;                                                       % [s]
% 
% % INTEGRATIVE LOOP *******************************
% Control.KI=160;                                                            % [N/Ns] Integrative gain
% % Control.Gi=Control.KI*G_T*K_A/kcond;                                       % [V/Ns]
% Control.Gi=0.02;                                                           % [V/Ns] Integrative gain
% Control.Dead_band=30;                                                      % [N] 
% 
% % PROPORTIONAL LOOP *******************************
% Control.KP=1;                                                              % [N/N] Propotional gain
% % Control.H3=Control.KP*G_T*K_A/kcond;                                       % [V/N] Propotional gain
% % Control.tauz3=0.022;                                                       % [s]
% % Control.taup3=0.022;                                                       % [s]
% Control.H3=0.001;                                                          % [V/N] Propotional gain
% Control.tauz3=0.009;                                                       % [s]
% Control.taup3=0.022;                                                       % [s]
% 
% % DERIVATIVE LOOP *************************
% Control.KD=3e-7;                                                           % [Ns/N] Derivative gain
% % Control.H4=Control.KD*K_A/kcond;                                           % [Vs/N] Derivative gain
% % Control.tauz4=0.1;                                                         % [s]  
% % Control.taup4=0.018;                                                       % [s]
% Control.H4=5e-6;                                                           % [Vs/N] Derivative gain
% Control.tauz4=0.1;                                                         % [s]  
% Control.taup4=0.018;                                                       % [s]
% 
% % ****************SPEED FEEDFORWARD *******************************
% Control.GFF=GFFgain;                                                       % [Ns/m]
% % Control.H5=Control.GFF*G_T*K_A/kcond;                                      % [Vs/m] Speed Feddforward gain
% % Control.taup5=0.00076929;                                                  % [s]
% % Control.tauz5=7.6929e-5;                                                   % [s]
% Control.H5=18;                                                              % [Vs/m]
% Control.taup5=0.00076929;                                                           % [s]
% Control.tauz5=7.6929e-5;                                                           % [s]
% Control.SV_delay=3e-3;                                                     % [s]  
% Control.kv=1;                                                              % [ms/ms]
% Control.ka=5e-6;                                                           % [ms^2/ms]
% 
% % **************Adaptative gain Y ******************************
% Control.expy=1.1;                                                          % [-]
% Control.deltap=0.5e6;                                                     % [Pa]
% Control.ky=0.01;                                                           % [-]
% 
% % Control.Hs=0;


% Z transform

% Tf_1=c2d(tf([Control.tauz1 1],[Control.taup1 1]),Sample,'foh');
% Control.tau_z1=get(Tf_1,'num');
% Control.tau_z1_1_2=Control.tau_z1{1,1};
% % Control.tau_z1_1_2=round(Control.tau_z1_1_2,6);
% Control.tau_p1=get(Tf_1,'den');
% Control.tau_p1_1_2=Control.tau_p1{1,1};
% % Control.tau_p1_1_2=round(Control.tau_p1_1_2,6);
% 
% Tf_3=c2d(tf([Control.tauz3 1],[Control.taup3 1]),Sample,'foh');
% Control.tau_z3=get(Tf_3,'num');
% Control.tau_z3_1_2=Control.tau_z3{1,1};
% % Control.tau_z3_1_2=round(Control.tau_z3_1_2,6);
% Control.tau_p3=get(Tf_3,'den');
% Control.tau_p3_1_2=Control.tau_p3{1,1};
% % Control.tau_p3_1_2=round(Control.tau_p3_1_2,6);
% 
% Tf_4=c2d(tf([Control.tauz4 1],[Control.taup4 1]),Sample,'foh');
% Control.tau_z4=get(Tf_4,'num');
% Control.tau_z4_1_2=Control.tau_z4{1,1};
% % Control.tau_z4_1_2=round(Control.tau_z4_1_2,6);
% Control.tau_p4=get(Tf_4,'den');
% Control.tau_p4_1_2=Control.tau_p4{1,1};
% % Control.tau_p4_1_2=round(Control.tau_p4_1_2,6);
% % 
% Tf_5=c2d(tf([Control.tauz5 1],[Control.taup5 1]),Sample,'foh');
% Control.tau_z5=get(Tf_5,'num');
% % Control.tau_z5_1_2=round(Control.tau_z5_1_2,6);
% Control.tau_p5=get(Tf_5,'den');
% Control.tau_p5_1_2=Control.tau_p5{1,1};
% Control.tau_z5_1_2=Control.tau_z5{1,1};
% % Control.tau_p5_1_2=round(Control.tau_p5_1_2,6);
% 
% Tf_6=c2d(tf([Control.tauz6 1],[Control.taup6 1]),Sample,'foh');
% Control.tau_z6=get(Tf_6,'num');
% Control.tau_z6_1_2=Control.tau_z6{1,1};
% % Control.tau_z6_1_2=round(Control.tau_z6_1_2,6);
% Control.tau_p6=get(Tf_6,'den');
% Control.tau_p6_1_2=Control.tau_p6{1,1};
% % Control.tau_p6_1_2=round(Control.tau_p6_1_2,6);




% Tf_1=c2d(tf([Control.tauz1 1],[Control.taup1 1]),1/5000,'foh');
% Control.tau_z1=Tf_1.Numerator{1,1};
% Control.tau_p1=Tf_1.Denominator{1,1};
% Tf_3=c2d(tf([Control.tauz3 1],[Control.taup3 1]),1/5000,'foh');
% Control.tau_z3=Tf_3.Numerator{1,1};
% Control.tau_p3=Tf_3.Denominator{1,1};
% Tf_4=c2d(tf([Control.tauz4 1],[Control.taup4 1]),1/5000,'foh');
% Control.tau_z4=Tf_4.Numerator{1,1};
% Control.tau_p4=Tf_4.Denominator{1,1};
% Tf_5=c2d(tf([Control.tauz5 1],[Control.taup5 1]),1/5000,'foh');
% Control.tau_z5=Tf_5.Numerator{1,1};
% Control.tau_p5=Tf_5.Denominator{1,1};
% Tf_6=c2d(tf([Control.tauz6 1],[Control.taup6 1]),1/5000,'foh');
% Control.tau_z6=Tf_6.Numerator{1,1};
% Control.tau_p6=Tf_6.Denominator{1,1};


% Selection mode initialization
% V_time=[0 0]';
% V_amplitude=[0 0]';
% TIME=0;
% SET=0;
% TIME_nc=0;
% SET_nc=0;
% External_velocity=0;
% V_command=0;
% warning ('off','all')
% Static=0;





